function output = MigrationLogLikelihood(params, times, states, distances)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Pagendam and Ross, 2011, Optimal GPS tracking for estimating species
%movements
%CODE for calculating the log-likelihood for a sequence of observations
%of the habitat patch that an individual is in.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%EXAMPLE for entering data
%params = [2.1, 1.5, 1/2];
%times = [1,2,5,6,10];
%states = [1, 1, 3, 4, 2]; %These are the patch numbers the individual
%was observed in.
%distances = [0 3 4 5; 3 0 1 6; 4 1 0 4; 5 6 4 0];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Q = SingleMigrationQ_3Param(distances, mu, delta, theta);
ll = 0;
ltm1 = length(times);
dimStates = size(states);
numTags = dimStates(1);
for(j = 1:numTags)
    for(i = 2:ltm1)
        t = times(i) - times(i-1);
        s1 = states(j, i-1);
        s2 = states(j, i);
        v = zeros(1, length(Q));
        v(s1) = 1;
        P = mexpv(t, Q', v);
        ll = ll + log(P(s2));
    end
end


output = -ll;