function I = SingleMigrationFIM_3Param(distances, mu, delta, theta, initial, times, threeOrTwo)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Pagendam and Ross, 2011, Optimal GPS tracking for estimating species
%movements
%CODE for forming Fisher Information Matrix for a single individual in a metapopulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%EXAMPLE for entering data
%distances = [0 3 4 5; 3 0 1 6; 4 1 0 4; 5 6 4 0]; %Pairwise distances between habitat patches.
%mu = 2;
%delta = 0.1;
%theta = 1/2;
%initial = 1; %The patch number to start in at time 0.
%times = [1 2 4 6.2]; %The times at which the individual's location/patch is observed.
%threeOrTwo = 1;If set to 1, find the optimal design for estimating all three parameters.  If set to 0, 
%find the optimal design for estimating mu and delta only.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Q = SingleMigrationQ_3Param(distances,mu,delta,theta);
lQ = length(Q);

for i = 1:lQ
    for j = 1:lQ
        if distances(i,j)>0
            logdist(i,j) = log(distances(i,j));
        else
            logdist(i,j) = 0;
        end
    end
end

%Compute the first and second order derivatives
pQmu = Q./mu; %SingleMigrationQ_3Param(distances,1,delta,theta);
pQdelta = -Q.*(distances.^theta); %SingleMigrationQ_3Param(distances,mu,delta,theta).*-(distances).^theta;
pQtheta = pQdelta.*logdist.*delta;
pQmumu = zeros(lQ, lQ);
pQmudelta = -(distances.^theta).*pQmu;
pQthetamu = pQtheta./mu;
pQdeltadelta = mu.*(distances.^(2*theta)).*pQmu;
pQthetadelta = pQtheta./delta - pQtheta.*(distances.^theta);
pQthetatheta = pQthetadelta.*delta.*logdist;

%Fix the derivatives on the diagonals
for(i=1:lQ)
    pQmu(i,i) = 0;
    pQmu(i,i) = -1*sum(pQmu(i,:));
    pQdelta(i,i) = 0;
    pQdelta(i,i) = -1*sum(pQdelta(i,:));
    pQmudelta(i,i) = 0;
    pQmudelta(i,i) = -1*sum(pQmudelta(i,:));
    pQmumu(i,i) = 0;
    pQmumu(i,i) = -1*sum(pQmumu(i,:));
    pQdeltadelta(i,i) = 0;
    pQdeltadelta(i,i) = -1*sum(pQdeltadelta(i,:));
	pQtheta(i,i) = 0;
	pQtheta(i,i) = -1*sum(pQtheta(i,:));
	pQthetamu(i,i) = 0;
	pQthetamu(i,i) = -1*sum(pQthetamu(i,:));
	pQthetadelta(i,i) = 0;
	pQthetadelta(i,i) = -1*sum(pQthetadelta(i,:));
	pQthetatheta(i,i) = 0;
	pQthetatheta(i,i) = -1*sum(pQthetatheta(i,:));
end

zM = zeros(size(Q));

ntimes = length(times);

Qs = [Q  pQmu pQdelta pQtheta pQmumu pQdeltadelta pQthetatheta pQmudelta  pQthetamu  pQthetadelta;
	  zM  Q      zM      zM   2*pQmu       zM          zM       pQdelta   pQtheta        zM;
	  zM zM      Q       zM   zM     2*pQdelta         zM       pQmu      zM         pQtheta;
	  zM zM      zM      Q    zM           zM      2*pQtheta     zM       pQmu       pQdelta;
	  zM zM      zM      zM   Q            zM          zM        zM       zM         zM;
	  zM zM      zM      zM   zM           Q           zM        zM       zM         zM;
	  zM zM      zM      zM   zM           zM          Q         zM       zM         zM;
	  zM zM      zM      zM   zM           zM          zM        Q        zM         zM;
	  zM zM      zM      zM   zM           zM          zM        zM       Q          zM;
	  zM zM      zM      zM   zM           zM          zM        zM       zM         Q];

iniv = zeros(ntimes - 1,10*lQ);
inivStart = zeros(1,lQ);
inivStart(initial) = 1;
%iniv(1,:) = [inivStart, zeros(1,9*lQ)];

I = zeros(3);

%for(i = 2:ntimes)
%    thisP =  mexpv(times(i-1),Q',iniv(1,1:lQ));
%    iniv(i,:) = [thisP', zeros(1,9*lQ)];
%end

Ident = eye(lQ);
diffTimes = diff([0,times]);
nDiffTimes = length(diffTimes);
for(j = 1:lQ)
        pv = expv(diffTimes(1),Qs',[Ident(j,:), zeros(1,9*lQ)]);
		I = I + inivStart(j)*[(1./pv(1:lQ))'*(pv(lQ+1:2*lQ).^2) - sum(pv(4*lQ+1:5*lQ)) (1./pv(1:lQ))'*(pv(lQ+1:2*lQ).*pv(2*lQ+1:3*lQ)) - sum(pv(7*lQ+1:8*lQ))  (1./pv(1:lQ))'*(pv(lQ+1:2*lQ).*pv(3*lQ+1:4*lQ)) - sum(pv(8*lQ+1:9*lQ)); (1./pv(1:lQ))'*(pv(lQ+1:2*lQ).*pv(2*lQ+1:3*lQ)) - sum(pv(7*lQ+1:8*lQ)) (1./pv(1:lQ))'*(pv(2*lQ+1:3*lQ).^2) - sum(pv(5*lQ+1:6*lQ)) (1./pv(1:lQ))'*(pv(3*lQ+1:4*lQ).*pv(2*lQ+1:3*lQ)) - sum(pv(9*lQ+1:10*lQ));  (1./pv(1:lQ))'*(pv(lQ+1:2*lQ).*pv(3*lQ+1:4*lQ)) - sum(pv(8*lQ+1:9*lQ))   (1./pv(1:lQ))'*(pv(3*lQ+1:4*lQ).*pv(2*lQ+1:3*lQ)) - sum(pv(9*lQ+1:10*lQ))   (1./pv(1:lQ))'*(pv(3*lQ+1:4*lQ).^2) - sum(pv(6*lQ+1:7*lQ))];
end
for(i = 2:nDiffTimes)
    thisP =  mexpv(times(i-1),Q',inivStart);
    iniv = [thisP', zeros(1,9*lQ)];
    for(j = 1:lQ)
        pv = expv(diffTimes(i),Qs',[Ident(j,:), zeros(1,9*lQ)]);
		I = I + iniv(j)*[(1./pv(1:lQ))'*(pv(lQ+1:2*lQ).^2) - sum(pv(4*lQ+1:5*lQ)) (1./pv(1:lQ))'*(pv(lQ+1:2*lQ).*pv(2*lQ+1:3*lQ)) - sum(pv(7*lQ+1:8*lQ))  (1./pv(1:lQ))'*(pv(lQ+1:2*lQ).*pv(3*lQ+1:4*lQ)) - sum(pv(8*lQ+1:9*lQ)); (1./pv(1:lQ))'*(pv(lQ+1:2*lQ).*pv(2*lQ+1:3*lQ)) - sum(pv(7*lQ+1:8*lQ)) (1./pv(1:lQ))'*(pv(2*lQ+1:3*lQ).^2) - sum(pv(5*lQ+1:6*lQ)) (1./pv(1:lQ))'*(pv(3*lQ+1:4*lQ).*pv(2*lQ+1:3*lQ)) - sum(pv(9*lQ+1:10*lQ));  (1./pv(1:lQ))'*(pv(lQ+1:2*lQ).*pv(3*lQ+1:4*lQ)) - sum(pv(8*lQ+1:9*lQ))   (1./pv(1:lQ))'*(pv(3*lQ+1:4*lQ).*pv(2*lQ+1:3*lQ)) - sum(pv(9*lQ+1:10*lQ))   (1./pv(1:lQ))'*(pv(3*lQ+1:4*lQ).^2) - sum(pv(6*lQ+1:7*lQ))];
    end
end

if threeOrTwo==1
    I = I(1:2,1:2);
end